@@ -18,21 +18,21 @@ class HTTPQuerySet(object):
     def _reset(self):
         self._result = None
 
-    def _raise(self, response):
+    def _exception(self, response):
         if response.status_code == 404:
-            raise ResourceNotFound(response.request.url)
+            raise ResourceNotFound(response)
         elif response.status_code == 409:
-            raise ResourceExists(response.request.url)
+            raise ResourceExists(response)
         elif response.status_code == 401:
-            raise UnAuthorized(response.request.url)
+            raise UnAuthorized(response)
         elif response.status_code == 403:
-            raise Forbidden(response.request.url)
+            raise Forbidden(response)
         elif response.status_code == 422:
             raise ValidationError(
-                json.loads(response.content)
+                json.loads(response)
             )
         else:
-            raise InvalidResource(response.content)
+            raise InvalidResource(response)
 
     def next(self):
         if not self._result:
@@ -40,71 +40,76 @@ class HTTPQuerySet(object):
 
         return self._resource(self._result.next())
 
-    def load(self, resource, url=None):
-        url = url or resource.url
-
+    def do_request(self, method, url, resource, **kwargs):
         try:
-            response = requests.get(url, headers=resource.headers)
+            response = requests.request(
+                method,
+                url,
+                headers=resource.headers,
+                **kwargs
+            )
         except requests.ConnectionError:
             raise ServiceUnavailable(url)
 
-        if response.status_code != 200:
-            raise self._raise(response)
+        if response.status_code not in (200, 201, 204):
+            raise self._exception(response)
 
         try:
-            data = json.loads(response.content)
+            return json.loads(response.content)
         except ValueError:
-            raise InvalidResource(response.content)
+            raise InvalidResource(response)
 
+    def load(self, resource, url=None):
+        url = url or resource.url
+
+        data = self.do_request('get', url, resource)
         resource.update(data)
 
     def insert(self, resource, validate=True):
-        response = requests.post(
+        data = self.do_request(
+            'post',
             resource.rel('create'),
+            resource,
             data=json.dumps(resource),
-            headers=resource.headers
         )
-
-        if response.status_code != 201:
-            self._raise(response)
-
-        try:
-            resource.clear()
-            resource.update(json.loads(response.content))
-        except ValueError, e:
-            raise InvalidResource(e)
+        resource.clear()
+        resource.update(data)
 
     def save(self, resource, validate=True, upsert=True):
-        response = requests.put(
+        self.do_request(
+            'put',
             resource.url,
             data=json.dumps(resource),
-            headers=resource.headers
         )
 
-        if response.status_code != 200:
-            self._raise(response)
-
     def patch(self, resource, patch):
-        response = requests.patch(
+        data = self.do_request(
+            'patch',
             resource.url,
-            headers=resource.headers,
+            resource,
             data=json.dumps(patch)
         )
 
-        if response.status_code != 200:
-            self._raise(response)
-
-        try:
-            resource.clear()
-            resource.update(json.loads(response.content))
-        except ValueError, e:
-            raise InvalidResource(e)
+        resource.clear()
+        resource.update(data)
 
     def delete(self, resource):
-        response = requests.delete(
+        self.do_request(
+            'delete',
             resource.url,
-            headers=resource.headers
         )
 
-        if response.status_code != 204:
-            self._raise(response)
+
+class AuthorizedQuerySet(HTTPQuerySet):
+    def do_request(self, method, url, resource, **kwargs):
+        try:
+            return super(AuthorizedQuerySet, self).do_request(
+                method, url, resource, **kwargs
+            )
+        except UnAuthorized:
+            if resource.token and resource.token.refresh():
+                return super(AuthorizedQuerySet, self).do_request(
+                    method, url, resource, **kwargs
+                )
+            else:
+                raise
